'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var lingxiEditorCore = require('lingxi-editor-core');
var lodash = require('lodash');
var React = require('react');
var React__default = _interopDefault(React);
var ReactDOM = _interopDefault(require('react-dom'));

const selectClassTarget = 'brickd-select-class-target';

const SPECIAL_STRING_CONSTANTS = {
    'null': null,
};
const formatSpecialProps = (props, propsConfig) => {
    const nextProps = props;
    lodash.each(props, (v, k) => {
        if (lodash.get(propsConfig, k)) {
            if (!lodash.isObject(v)) {
                if (SPECIAL_STRING_CONSTANTS[v] !== undefined) {
                    nextProps[k] = SPECIAL_STRING_CONSTANTS[v];
                }
                else if (propsConfig[k].type === lingxiEditorCore.PROPS_TYPES.function) {
                    const funcTemplate = lodash.get(propsConfig, `${k}.placeholder`);
                    if (funcTemplate) {
                        nextProps[k] = () => eval(funcTemplate);
                    }
                    else {
                        nextProps[k] = () => void 0;
                    }
                }
            }
            else if (lodash.isObject(v) && !lodash.isEmpty(propsConfig[k].childPropsConfig) && lodash.isEqual(lodash.keys(v), lodash.keys(propsConfig[k].childPropsConfig))) {
                formatSpecialProps(v, propsConfig[k].childPropsConfig);
            }
        }
        else if (lodash.isUndefined(v)) {
            delete nextProps[k];
        }
    });
    return nextProps;
};
function usePrevious(value) {
    const ref = React.useRef();
    React.useEffect(() => {
        ref.current = value;
    });
    return ref.current;
}
const iframeSrcDoc = `<!DOCTYPE html>
<html lang="en">
<meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=0.5, maximum-scale=2.0, user-scalable=yes" />
<body class="theme-mall-science">
<div id="dnd-container" style="width: 100%;height: 100%"></div>
</body>
</html>
`;
const isEqualKey = (key, selectKey) => {
    if (!selectKey)
        return false;
    return selectKey.includes(key) && parseInt(selectKey) === parseInt(key);
};
const getIframe = () => {
    return document.getElementById('dnd-iframe');
};
const getComponent = (componentName) => lodash.get(lingxiEditorCore.LEGO_BRIDGE.config.OriginalComponents, componentName, componentName);
function formatUnit(target) {
    if (target) {
        const result = target.match(/\d+/);
        if (result) {
            return Number.parseInt(result[0]);
        }
    }
    return null;
}
const getSelectedNode = (key, iframe) => {
    if (iframe && key) {
        const { contentDocument } = iframe;
        return contentDocument.getElementsByClassName(selectClassTarget + parseInt(key))[0];
    }
};
function generateCSS(left, top, width, height) {
    return `
    ${width ? `width:${width}px;` : ''}
    ${height ? `height:${height}px;` : ''}
    display:flex;
    left:${left}px;
    top:${top}px;
  `;
}
function getElementInfo(element, iframe, isModal) {
    const { contentWindow } = iframe;
    const { scrollX, scrollY } = contentWindow;
    const { width, height, left, top } = element.getBoundingClientRect();
    let newLeft = left;
    let newTop = top;
    if (!isModal) {
        newLeft += scrollX;
        newTop += scrollY;
    }
    return { width, height, left: newLeft, top: newTop, bottom: newTop + height, right: newLeft + width };
}
function getIsModalChild(componentConfigs, domTreeKeys) {
    if (domTreeKeys) {
        for (const key of domTreeKeys) {
            const { mirrorModalField } = lingxiEditorCore.getComponentConfig(lodash.get(componentConfigs, [key, 'componentName'])) || {};
            if (mirrorModalField)
                return true;
        }
    }
    return false;
}
function setPosition(nodes, isModal) {
    if (isModal) {
        lodash.each(nodes, (node) => node.style.position = 'fixed');
    }
    else {
        lodash.each(nodes, (node) => node.style.position = 'absolute');
    }
}
function generateRequiredProps(componentName) {
    const { propsConfig } = lingxiEditorCore.getComponentConfig(componentName);
    const requiredProps = {};
    lodash.each(propsConfig, (config, propName) => {
        const { isRequired, defaultValue } = config;
        if (isRequired)
            requiredProps[propName] = defaultValue;
    });
    return requiredProps;
}

function handleSelectedStatus(event, isSelected, specialProps, propName) {
    event && event.stopPropagation && event.stopPropagation();
    if (isSelected) {
        lingxiEditorCore.clearSelectedStatus();
    }
    else {
        lingxiEditorCore.selectComponent({ ...specialProps, propName });
    }
}
/**
 * hover组件上触发
 * @param event
 * @param key
 */
function onMouseOver(event, key, isSelected) {
    event && event.stopPropagation && event.stopPropagation();
    if (isSelected) {
        lingxiEditorCore.clearHovered();
    }
    else {
        lingxiEditorCore.overTarget({
            hoverKey: key,
        });
    }
}
/**
 * 获取要放入组件的容器信息
 * @param event
 * @param domTreeKeys
 * @param selectedKey
 * @param propName
 */
function getDropTargetInfo(event, domTreeKeys, selectedKey, propName) {
    event && event.stopPropagation && event.stopPropagation();
    lingxiEditorCore.getDropTarget({
        propName,
        selectedKey,
        domTreeKeys,
    });
}
/**
 * 拖拽当前组件时获取当前组件的信息
 * @param event
 * @param dragKey
 * @param parentKey
 * @param parentPropName
 */
function onDragStart(event, dragKey, parentKey, parentPropName) {
    event && event.stopPropagation && event.stopPropagation();
    setTimeout(() => {
        lingxiEditorCore.getDragSource({
            dragKey,
            parentKey,
            parentPropName,
        });
    }, 0);
}
/**
 * 获取组件选中状态
 * @param key
 * @param hoverKey
 * @param selectedKey
 */
function selectedStatus(key, hoverKey, selectedKey) {
    const isSelected = isEqualKey(key, selectedKey);
    /** 是否hover到当前组件 */
    const isHovered = isEqualKey(key, hoverKey);
    return { isHovered, isSelected };
}
function onDragover(e) {
    e.preventDefault();
}
function onDrop(e) {
    e.stopPropagation();
    lingxiEditorCore.addComponent();
}

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') {
    return;
  }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".style_animate__3xdr0 {\n  transition: all 50ms;\n}\n.style_distances-tip__2G9PV {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n}\n.style_forbid-event__2_6y0 {\n  pointer-events: none;\n}\n.style_allow-event__SEvBa {\n  pointer-events: auto;\n}\n.style_hidden-component__1J6xk {\n  display: none;\n  transition: all 700ms;\n}\n";
var styles = {"animate":"style_animate__3xdr0","distances-tip":"style_distances-tip__2G9PV","forbid-event":"style_forbid-event__2_6y0","allow-event":"style_allow-event__SEvBa","hidden-component":"style_hidden-component__1J6xk"};
styleInject(css_248z);

function controlUpdate(prevState, nextState, key, isModal) {
    const { selectedKey: prevSelectedKey, propName: prevPropName, domTreeKeys: prevDomTreeKeys } = prevState.selectedInfo || {};
    const { selectedKey, propName, domTreeKeys } = nextState.selectedInfo || {};
    if (!prevSelectedKey && selectedKey) {
        if (isModal) {
            if (domTreeKeys.includes(key)) {
                return true;
            }
        }
        return isEqualKey(key, selectedKey);
    }
    if (prevSelectedKey && !selectedKey) {
        if (isModal) {
            if (prevDomTreeKeys.includes(key)) {
                return true;
            }
        }
        return isEqualKey(key, prevSelectedKey);
    }
    if (prevSelectedKey && selectedKey) {
        if (isModal) {
            if (!prevDomTreeKeys.includes(key) && domTreeKeys.includes(key) || prevDomTreeKeys.includes(key) && !domTreeKeys.includes(key)) {
                return true;
            }
        }
        if (prevSelectedKey !== selectedKey) {
            return isEqualKey(key, prevSelectedKey) || !isEqualKey(key, prevSelectedKey) && isEqualKey(key, selectedKey);
        }
        else {
            return propName !== prevPropName;
        }
    }
    return false;
}
function useSelect(specialProps, isModal) {
    const { key } = specialProps;
    const { selectedInfo } = lingxiEditorCore.useSelector(['selectedInfo'], (prevState, nextState) => controlUpdate(prevState, nextState, key, isModal));
    const { selectedKey, domTreeKeys: selectedDomKeys } = selectedInfo || {};
    const isSelected = isEqualKey(key, selectedKey);
    React.useEffect(() => {
        if (isSelected) {
            handleSelectedStatus(null, false, specialProps);
        }
    }, []);
    return { selectedDomKeys, isSelected };
}

function dragDropUpdate(prevState, nextState, key) {
    const { selectedKey, domTreeKeys } = nextState.dropTarget || {};
    const { selectedKey: prevSelectedKey, domTreeKeys: prevDomTreeKeys } = prevState.dropTarget || {};
    const { dragKey, parentKey } = nextState.dragSource || {};
    const { dragKey: prevDragKey } = prevState.dragSource || {};
    //drop相关 分立不合并
    if (prevSelectedKey === key && selectedKey !== key || prevSelectedKey !== key && selectedKey === key) {
        return true;
    }
    if (prevDragKey === dragKey && dragKey == key && parentKey) {
        if (!prevDomTreeKeys && domTreeKeys) {
            return domTreeKeys.includes(parentKey);
        }
        if (prevDomTreeKeys && !domTreeKeys) {
            return prevDomTreeKeys.includes(parentKey);
        }
        if (prevDomTreeKeys && domTreeKeys) {
            return prevDomTreeKeys.includes(parentKey) && !domTreeKeys.includes(parentKey) || !prevDomTreeKeys.includes(parentKey) && domTreeKeys.includes(parentKey);
        }
    }
    //drag相关 分立不合并
    return prevDragKey && !dragKey || !prevDragKey && !!dragKey || prevDragKey === key && dragKey !== key || prevDragKey !== key && dragKey === key;
}
function useDragDrop(key) {
    const { dragSource, dropTarget } = lingxiEditorCore.useSelector(['dragSource', 'dropTarget'], (prevState, nextState) => dragDropUpdate(prevState, nextState, key));
    const { selectedKey, domTreeKeys } = dropTarget || {};
    const { parentKey, dragKey } = dragSource || {};
    const isHidden = dragKey === key && !!parentKey && !!domTreeKeys && parentKey !== selectedKey && !domTreeKeys.includes(parentKey);
    return { dragSource, dropTarget, isHidden };
}

function NoneContainer(allProps, ref) {
    const { specialProps, specialProps: { key, domTreeKeys, draggable }, isDragAddChild, ...rest } = allProps;
    const { componentConfigs: PageDom, propsConfigSheet } = lingxiEditorCore.useSelector(stateSelector, (prevState, nextState) => controlUpdate$1(prevState, nextState, key));
    const { isSelected } = useSelect(specialProps);
    const { dragSource, isHidden } = useDragDrop(key);
    const { dragKey, vDOMCollection } = dragSource || {};
    const componentConfigs = PageDom[lingxiEditorCore.ROOT] ? PageDom : vDOMCollection || {};
    const { props, componentName } = componentConfigs[key] || {};
    const { propsConfig } = React.useMemo(() => lingxiEditorCore.getComponentConfig(componentName), []);
    if (!componentName)
        return null;
    const onDragEnter = (e) => {
        e.stopPropagation();
        if (dragKey && domTreeKeys.includes(dragKey)) {
            lingxiEditorCore.clearDropTarget();
        }
    };
    const { className, animateClass, ...restProps } = props || {};
    return (React.createElement(getComponent(componentName), {
        ...restProps,
        className: handlePropsClassName(key, isHidden && !isDragAddChild, dragKey === key, className, animateClass),
        ...(isDragAddChild ? {} : {
            onDragEnter,
            ...handleEvents(specialProps, isSelected),
        }),
        ...generateRequiredProps(componentName),
        ...formatSpecialProps(props, lingxiEditorCore.produce(propsConfig, oldPropsConfig => {
            lodash.merge(oldPropsConfig, propsConfigSheet[specialProps.key]);
        })),
        draggable,
        /**
         * 设置组件id方便抓取图片
         */
        ref,
        ...rest,
    }));
}
var NoneContainer$1 = React.memo(React.forwardRef(NoneContainer), propAreEqual);

/**
 * 处理样式
 * @param key
 * @param isHidden
 * @param isDragTarget
 * @param className
 * @param animateClass
 * @param isDropTarget
 */
function handlePropsClassName(key, isHidden, isDragTarget, className, animateClass) {
    return `${selectClassTarget + key} ${className} ${animateClass} ${isDragTarget ? styles['forbid-event'] : styles['allow-event']} ${isHidden && styles['hidden-component']}`;
}
/**
 * 渲染组件的子节点
 * @param childNodes
 * @param specialProps
 * @param componentConfigs
 * @param dragAddStatus
 * @param parentPropName
 * @param isOnlyNode
 */
function renderNodes(childNodes, specialProps, componentConfigs, dragAddStatus, parentPropName, isOnlyNode) {
    const { dragKey, isDragAdd, isDragAddChild } = dragAddStatus;
    let { domTreeKeys } = specialProps;
    const parentKey = specialProps.key;
    if (parentPropName) {
        domTreeKeys = [...domTreeKeys, `${parentKey}${parentPropName}`];
    }
    const resultChildNodes = lodash.map(childNodes, (key) => {
        const { componentName } = componentConfigs[key] || {};
        if (!componentName)
            return null;
        /** 根据组件类型处理属性 */
        const props = {
            specialProps: {
                key,
                domTreeKeys: [...domTreeKeys, key],
                parentKey,
                parentPropName,
            },
        };
        return lingxiEditorCore.isContainer(componentName) ?
            React__default.createElement(Container$1, Object.assign({}, props, { isDragAddChild: isDragAddChild || dragKey === key && isDragAdd, key: key })) :
            React__default.createElement(NoneContainer$1, Object.assign({}, props, { isDragAddChild: isDragAddChild || dragKey === key && isDragAdd, key: key }));
    });
    /** 如果该组件子节点或者属性子节点要求为单组件返回子组件的第一组件*/
    if (isOnlyNode) {
        return resultChildNodes[0];
    }
    return resultChildNodes;
}
function renderRequiredChildNodes(childNodesRule) {
    if (childNodesRule) {
        const componentName = childNodesRule[0];
        return React.createElement(getComponent(componentName), generateRequiredProps(componentName));
    }
    else {
        return React__default.createElement("div", null);
    }
}
function handleRequiredChildNodes(componentName) {
    const { isRequired, nodePropsConfig, childNodesRule } = lingxiEditorCore.getComponentConfig(componentName);
    const nodeProps = {};
    if (isRequired) {
        nodeProps.children = renderRequiredChildNodes(childNodesRule);
    }
    else if (nodePropsConfig) {
        lodash.each(nodePropsConfig, (propConfig, key) => {
            const { isRequired, childNodesRule } = propConfig;
            if (isRequired) {
                nodeProps[key] = renderRequiredChildNodes(childNodesRule);
            }
        });
    }
    return nodeProps;
}
function handleChildNodes(specialProps, componentConfigs, children, dragKey, isDragAdd, isDragAddChild) {
    const nodeProps = {};
    const { key: parentKey } = specialProps;
    const { componentName } = componentConfigs[parentKey];
    if (lodash.isEmpty(children)) {
        return handleRequiredChildNodes(componentName);
    }
    const dragAddStatus = { isDragAdd, dragKey, isDragAddChild };
    const { nodePropsConfig, isOnlyNode } = lingxiEditorCore.getComponentConfig(componentName);
    if (Array.isArray(children)) {
        nodeProps.children = renderNodes(children, specialProps, componentConfigs, dragAddStatus, undefined, isOnlyNode);
    }
    else {
        lodash.each(children, (nodes, propName) => {
            const { isOnlyNode, isRequired, childNodesRule } = nodePropsConfig[propName];
            if (lodash.isEmpty(nodes))
                return isRequired && (nodeProps[propName] = renderRequiredChildNodes(childNodesRule));
            nodeProps[propName] = renderNodes(nodes, specialProps, componentConfigs, dragAddStatus, propName, isOnlyNode);
        });
    }
    return nodeProps;
}
/**
 * 处理弹窗类容器
 * @param mirrorModalField
 * @param iframeId
 */
function handleModalTypeContainer(mirrorModalField) {
    const mountedProps = {};
    const { displayPropName, mounted } = mirrorModalField;
    if (mounted) {
        const { propName, type } = mounted;
        const iframe = getIframe();
        const mountedNode = iframe.contentDocument.body;
        mountedProps[propName] = type === lingxiEditorCore.PROPS_TYPES.function ? () => mountedNode : mountedNode;
    }
    return { displayPropName, mountedProps };
}
/**
 * 处理容器组件属性
 * @returns {*|{onMouseOver: (function(*=): void), onClick: (function(*=): *), onDragStart: (function(*=): void), onDragEnter: (function(*=): *)}}
 */
function handleEvents(specialProps, isSelected, componentName, childNodes) {
    const { key, parentKey, parentPropName, } = specialProps;
    let propName;
    if (childNodes && !Array.isArray(childNodes)) {
        propName = Object.keys(childNodes)[0];
    }
    return {
        onClick: (e) => handleSelectedStatus(e, isSelected, specialProps, propName),
        onMouseOver: (e) => onMouseOver(e, key, isSelected),
        onDragStart: (e) => onDragStart(e, key, parentKey, parentPropName),
        onDragEnd: () => lingxiEditorCore.clearDragSource()
    };
}
const stateSelector = ['componentConfigs', 'propsConfigSheet'];
function controlUpdate$1(prevState, nextState, key) {
    return prevState.componentConfigs[key] !== nextState.componentConfigs[key];
}
function propAreEqual(prevProps, nextProps) {
    const { specialProps: prevSpecialProps, isDragAddChild: prevIsDragAddChild, ...prevRest } = prevProps;
    const { specialProps, isDragAddChild, ...rest } = nextProps;
    return lodash.isEqual(prevRest, rest) && lodash.isEqual(prevSpecialProps, specialProps) && prevIsDragAddChild === isDragAddChild;
}

function useChildNodes({ specialProps, childNodes, componentName }) {
    const { nodePropsConfig, isRequired } = React.useMemo(() => lingxiEditorCore.getComponentConfig(componentName), []);
    React.useEffect(() => {
        if (!childNodes)
            return;
        if (!Array.isArray(childNodes)) {
            for (const prop of Object.keys(nodePropsConfig)) {
                const { isRequired } = nodePropsConfig[prop];
                if (isRequired && childNodes[prop].length === 0) {
                    handleSelectedStatus(null, false, specialProps, prop);
                    break;
                }
            }
        }
        else if (isRequired && childNodes.length === 0) {
            handleSelectedStatus(null, false, specialProps);
        }
    }, [childNodes]);
}

/**
 * 所有的容器组件名称
 */
function Container(allProps, ref) {
    const { specialProps, specialProps: { key, domTreeKeys, draggable }, isDragAddChild, ...rest } = allProps;
    const { componentConfigs: PageDom, propsConfigSheet } = lingxiEditorCore.useSelector(stateSelector, (prevState, nextState) => controlUpdate$1(prevState, nextState, key));
    const { dragSource, dropTarget, isHidden } = useDragDrop(key);
    const { dragKey, parentKey, vDOMCollection } = dragSource || {};
    const { selectedKey } = dropTarget || {};
    const componentConfigs = PageDom[lingxiEditorCore.ROOT] ? PageDom : vDOMCollection || {};
    const { props, childNodes, componentName } = componentConfigs[key] || {};
    useChildNodes({ childNodes, componentName, specialProps });
    const [children, setChildren] = React.useState(childNodes);
    const { mirrorModalField, propsConfig, nodePropsConfig } = React.useMemo(() => lingxiEditorCore.getComponentConfig(componentName), []);
    const { selectedDomKeys, isSelected } = useSelect(specialProps, !!mirrorModalField);
    const onDragEnter = (e) => {
        e.stopPropagation();
        let propName;
        //判断当前组件是否为拖拽组件的父组件或者是否为子孙组件或者它自己
        if (parentKey !== key &&
            dragKey &&
            dragKey !== key &&
            !domTreeKeys.includes(dragKey) &&
            (!selectedDomKeys || !selectedDomKeys.includes(dragKey))) {
            if (childNodes) {
                if (Array.isArray(childNodes)) {
                    if (!lingxiEditorCore.handleRules(componentConfigs, dragKey, key, undefined, true)) {
                        setChildren([...childNodes, dragKey]);
                    }
                }
                else {
                    setChildren(lingxiEditorCore.produce(childNodes, oldChild => {
                        propName = Object.keys(oldChild)[0];
                        if (!lingxiEditorCore.handleRules(componentConfigs, dragKey, key, propName, true)) {
                            oldChild[propName] = [...oldChild[propName], dragKey];
                        }
                    }));
                }
            }
            else {
                if (!nodePropsConfig) {
                    setChildren([dragKey]);
                }
                else {
                    const keys = Object.keys(nodePropsConfig);
                    propName = keys[0];
                    setChildren({ [propName]: [dragKey] });
                }
            }
        }
        getDropTargetInfo(e, domTreeKeys, key, propName);
    };
    const onDragLeave = (e) => {
        e.stopPropagation();
        if (isSelected) {
            lingxiEditorCore.clearDropTarget();
        }
    };
    React.useEffect(() => {
        setChildren(childNodes);
    }, [childNodes]);
    if ((!selectedKey || selectedKey !== key) && children !== childNodes) {
        setChildren(childNodes);
    }
    if (!componentName)
        return null;
    let modalProps = {};
    if (mirrorModalField) {
        const { displayPropName, mountedProps } = handleModalTypeContainer(mirrorModalField);
        const isVisible = isSelected || selectedDomKeys && selectedDomKeys.includes(key);
        modalProps = { [displayPropName]: isVisible, ...mountedProps };
    }
    const { className, animateClass, ...restProps } = props || {};
    return React.createElement(getComponent(componentName), {
        ...restProps,
        className: handlePropsClassName(key, isHidden && !isDragAddChild, dragKey === key, className, animateClass),
        ...(isDragAddChild ? {} : {
            onDragEnter,
            onDragLeave,
            ...handleEvents(specialProps, isSelected, componentName, childNodes),
        }),
        ...generateRequiredProps(componentName),
        ...handleChildNodes(specialProps, componentConfigs, children, dragKey, childNodes !== children, isDragAddChild),
        ...formatSpecialProps(props, lingxiEditorCore.produce(propsConfig, oldPropsConfig => {
            lodash.merge(oldPropsConfig, propsConfigSheet[key]);
        })),
        draggable,
        /**
         * 设置组件id方便抓取图片
         */
        ref,
        ...rest,
        ...modalProps,
    });
}
var Container$1 = React.memo(React.forwardRef(Container), propAreEqual);

var css_248z$1 = ".index_animate__3IMwR {\n  transition: all 50ms;\n}\n.index_distances-tip__2tQqO {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n}\n.index_guide-common__1uq62 {\n  position: absolute;\n  z-index: 10000;\n  top: 0;\n  transition: all 50ms;\n}\n.index_guide-v__RgPZj {\n  position: absolute;\n  z-index: 10000;\n  top: 0;\n  transition: all 50ms;\n  height: 100%;\n  border-left: 1px dashed #00B37A;\n}\n.index_guide-h__2bn6t {\n  position: absolute;\n  z-index: 10000;\n  top: 0;\n  transition: all 50ms;\n  width: 100%;\n  border-top: 1px dashed #00B37A;\n}\n.index_guide-hidden__2JF7F {\n  display: none;\n  position: absolute;\n  z-index: -1;\n}\n.index_hover-node__2PNv6 {\n  position: absolute;\n  z-index: 10000;\n  top: 0;\n  transition: all 50ms;\n  border: 1px solid #00B37A;\n  pointer-events: none;\n  background-color: rgba(0, 0, 0, 0.1);\n}\n.index_drop-node__2PkO_ {\n  position: absolute;\n  z-index: 10000;\n  top: 0;\n  transition: all 50ms;\n  border: 1px solid #00B37A;\n  pointer-events: none;\n  background-color: rgba(0, 0, 0, 0.1);\n  border-color: red ;\n  background-color: rgba(255, 0, 0, 0.2);\n}\n";
var styles$1 = {"animate":"index_animate__3IMwR","distances-tip":"index_distances-tip__2tQqO","guide-common":"index_guide-common__1uq62","guide-v":"index_guide-v__RgPZj","guide-h":"index_guide-h__2bn6t","guide-hidden":"index_guide-hidden__2JF7F","hover-node":"index_hover-node__2PNv6","drop-node":"index_drop-node__2PkO_"};
styleInject(css_248z$1);

const controlUpdate$2 = () => {
    return true;
};
function Guidelines() {
    const topRef = React.useRef();
    const bottomRef = React.useRef();
    const leftRef = React.useRef();
    const rightRef = React.useRef();
    const hoverNodeRef = React.useRef();
    const iframe = getIframe();
    const { hoverKey, dropTarget, dragSource, selectedInfo, componentConfigs } = lingxiEditorCore.useSelector(['hoverKey', 'dropTarget', 'dragSource', 'selectedInfo', 'componentConfigs'], controlUpdate$2);
    const { domTreeKeys } = selectedInfo || {};
    const isModal = getIsModalChild(componentConfigs, domTreeKeys);
    const guidControl = hoverKey && (!selectedInfo || selectedInfo && !dragSource);
    React.useEffect(() => {
        const { contentWindow } = iframe;
        const renderGuideLines = () => {
            const node = getSelectedNode(hoverKey, iframe);
            if (guidControl && node) {
                const { left, top, bottom, right, width, height } = getElementInfo(node, iframe, isModal);
                const { contentDocument } = iframe;
                hoverNodeRef.current.style.cssText = generateCSS(left, top, width, height);
                topRef.current.style.top = `${top}px`;
                topRef.current.style.width = `${contentDocument.body.scrollWidth}px`;
                leftRef.current.style.left = `${left}px`;
                leftRef.current.style.height = `${contentDocument.body.scrollHeight}px`;
                rightRef.current.style.left = `${right - 1}px`;
                rightRef.current.style.height = `${contentDocument.body.scrollHeight}px`;
                bottomRef.current.style.top = `${bottom - 1}px`;
                bottomRef.current.style.width = `${contentDocument.body.scrollWidth}px`;
                setPosition([hoverNodeRef.current, leftRef.current, rightRef.current, topRef.current, bottomRef.current], isModal);
            }
        };
        renderGuideLines();
        const onScroll = () => {
            setTimeout(renderGuideLines, 66);
        };
        contentWindow.addEventListener('scroll', onScroll);
        return () => {
            contentWindow.removeEventListener('scroll', onScroll);
        };
    });
    const guidH = guidControl ? styles$1['guide-h'] : styles$1['guide-hidden'];
    const guidV = guidControl ? styles$1['guide-v'] : styles$1['guide-hidden'];
    const hoverNode = guidControl ? dropTarget ? styles$1['drop-node'] : styles$1['hover-node'] : styles$1['guide-hidden'];
    return (React__default.createElement(React__default.Fragment, null,
        React__default.createElement("div", { ref: hoverNodeRef, className: hoverNode }),
        React__default.createElement("div", { ref: leftRef, className: guidV }),
        React__default.createElement("div", { ref: rightRef, className: guidV }),
        React__default.createElement("div", { ref: topRef, className: guidH }),
        React__default.createElement("div", { ref: bottomRef, className: guidH })));
}
var Guidelines$1 = React.memo(Guidelines);

var css_248z$2 = ".index_animate__1Ri1Z {\n  transition: all 50ms;\n}\n.index_distances-tip__2Lhv- {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n}\n.index_distances-common__2ixMw {\n  position: absolute;\n  z-index: 30000;\n  display: none;\n  justify-content: center;\n  align-items: center;\n  pointer-events: none;\n  transition: all 50ms;\n}\n.index_distances-v__3ZvOg {\n  position: absolute;\n  z-index: 30000;\n  display: none;\n  justify-content: center;\n  align-items: center;\n  pointer-events: none;\n  transition: all 50ms;\n  width: 1px;\n  height: 100%;\n}\n.index_distances-v__3ZvOg:before {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n  content: attr(data-distance);\n  pointer-events: none;\n  left: 5px;\n}\n.index_distances-h__3OAtM {\n  position: absolute;\n  z-index: 30000;\n  display: none;\n  justify-content: center;\n  align-items: center;\n  pointer-events: none;\n  transition: all 50ms;\n  height: 1px;\n  width: 100%;\n}\n.index_distances-h__3OAtM:before {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n  content: attr(data-distance);\n  pointer-events: none;\n  top: -20px;\n}\n";
var styles$2 = {"animate":"index_animate__1Ri1Z","distances-tip":"index_distances-tip__2Lhv-","distances-common":"index_distances-common__2ixMw","distances-v":"index_distances-v__3ZvOg","distances-h":"index_distances-h__3OAtM"};
styleInject(css_248z$2);

function handleDistances(selectRect, hoverRect) {
    const { left: selectLeft, top: selectTop, bottom: selectBottom, right: selectRight, width: selectWidth, height: selectHeight } = selectRect;
    const { left, top, bottom, right, width, height } = hoverRect;
    let leftDistance = 0;
    let leftGuide = 0;
    let rightDistance = 0;
    let rightGuide = 0;
    let topDistance = 0;
    let topGuide = 0;
    let bottomDistance = 0;
    let bottomGuide = 0;
    leftDistance = selectLeft - left;
    rightDistance = selectRight - right;
    topDistance = selectTop - top;
    bottomDistance = selectBottom - bottom;
    //select组件左右边框在hover组件内部
    if (width > selectWidth) {
        if (leftDistance == 0) {
            rightGuide = selectRight;
        }
        else if (rightDistance === 0) {
            leftGuide = left;
        }
        else if (leftDistance > 0 && rightDistance < 0) {
            leftGuide = left;
            rightGuide = selectRight;
        }
        else if (leftDistance < 0 && Math.abs(leftDistance) < selectWidth) { //select组件右侧出hover组件
            leftGuide = selectLeft;
            rightGuide = selectRight;
        }
        else if (leftDistance < 0 && Math.abs(leftDistance) > selectWidth) { //select组件
            leftDistance = 0;
            rightDistance = left - selectRight;
            rightGuide = selectRight;
        }
        else if (rightDistance > 0 && rightDistance < selectWidth) { //select组件左侧出hover组件
            leftGuide = left;
            rightGuide = right;
        }
        else if (rightDistance > 0 && rightDistance > selectWidth) {
            rightDistance = 0;
            leftDistance = selectLeft - right;
            leftGuide = right;
        }
        else if (Math.abs(leftDistance) === selectWidth || rightDistance === selectWidth) {
            leftDistance = 0;
            rightDistance = 0;
        }
    }
    else {
        if (leftDistance == 0) {
            rightGuide = right;
        }
        else if (rightDistance === 0) {
            leftGuide = selectLeft;
        }
        else if (leftDistance < 0 && rightDistance > 0) {
            leftGuide = selectLeft;
            rightGuide = right;
        }
        else if (leftDistance > 0 && leftDistance < width) {
            leftGuide = left;
            rightGuide = right;
        }
        else if (leftDistance > 0 && leftDistance > width) {
            rightDistance = 0;
            leftDistance = selectLeft - right;
            leftGuide = right;
        }
        else if (rightDistance < 0 && Math.abs(rightDistance) < width) {
            leftGuide = selectLeft;
            rightGuide = selectRight;
        }
        else if (rightDistance < 0 && Math.abs(rightDistance) > width) {
            leftDistance = 0;
            rightDistance = selectRight - left;
            rightGuide = selectRight;
        }
        else if (leftDistance === width || Math.abs(rightDistance) === width) {
            rightDistance = 0;
            leftDistance = 0;
        }
    }
    if (height > selectHeight) {
        if (topDistance === 0) {
            bottomGuide = selectBottom;
        }
        else if (bottomDistance === 0) {
            topGuide = top;
        }
        else if (topDistance > 0 && bottomDistance < 0) {
            topGuide = top;
            bottomGuide = selectBottom;
        }
        else if (topDistance < 0 && Math.abs(topDistance) < selectHeight) {
            topGuide = selectTop;
            bottomGuide = selectBottom;
        }
        else if (topDistance < 0 && Math.abs(topDistance) > selectHeight) {
            topDistance = 0;
            bottomDistance = selectBottom - top;
            bottomGuide = selectBottom;
        }
        else if (bottomDistance > 0 && bottomDistance < selectHeight) {
            topGuide = top;
            bottomGuide = bottom;
        }
        else if (bottomDistance > 0 && bottomDistance > selectHeight) {
            bottomDistance = 0;
            topDistance = selectTop - bottom;
            topGuide = bottom;
        }
        else if (Math.abs(topDistance) === selectHeight || bottomDistance === selectHeight) {
            bottomDistance = 0;
            topDistance = 0;
        }
    }
    else {
        if (topDistance === 0) {
            bottomGuide = bottom;
        }
        else if (bottomDistance === 0) {
            topGuide = selectTop;
        }
        else if (topDistance < 0 && bottomDistance > 0) {
            topGuide = selectTop;
            bottomGuide = bottom;
        }
        else if (topDistance > 0 && topDistance < height) {
            topGuide = top;
            bottomGuide = bottom;
        }
        else if (topDistance > 0 && topDistance > height) {
            bottomDistance = 0;
            topDistance = selectTop - bottom;
            topGuide = bottom;
        }
        else if (bottomDistance < 0 && Math.abs(bottomDistance) < height) {
            topGuide = selectTop;
            bottomGuide = selectBottom;
        }
        else if (bottomDistance < 0 && Math.abs(bottomDistance) > height) {
            topDistance = 0;
            bottomDistance = selectBottom - top;
            bottomGuide = selectBottom;
        }
        else if (topDistance === height || Math.abs(bottomDistance) === height) {
            bottomDistance = 0;
            topDistance = 0;
        }
    }
    const result = {};
    lodash.each({
        leftGuide,
        leftDistance,
        rightDistance,
        rightGuide,
        topDistance,
        topGuide,
        bottomGuide,
        bottomDistance,
    }, (v, k) => result[k] = Math.round(Math.abs(v)));
    return result;
}
function Distances() {
    const topRef = React.useRef();
    const bottomRef = React.useRef();
    const leftRef = React.useRef();
    const rightRef = React.useRef();
    const iframe = getIframe();
    const { hoverKey, selectedInfo, dragSource, componentConfigs } = lingxiEditorCore.useSelector(['hoverKey', 'selectedInfo', 'dragSource', 'componentConfigs']);
    const { selectedKey, domTreeKeys } = selectedInfo || {};
    const isModal = getIsModalChild(componentConfigs, domTreeKeys);
    const hoverNode = getSelectedNode(hoverKey, iframe);
    const selectNode = React.useMemo(() => getSelectedNode(selectedKey, iframe), [selectedKey, iframe]);
    if (!dragSource && hoverNode && selectNode) {
        const selectRect = getElementInfo(selectNode, iframe, isModal);
        const { width, height, top, left } = selectRect;
        const hoverRect = getElementInfo(hoverNode, iframe, isModal);
        const { leftGuide, leftDistance, rightDistance, rightGuide, topDistance, topGuide, bottomGuide, bottomDistance } = handleDistances(selectRect, hoverRect);
        if (leftDistance !== 0) {
            leftRef.current.style.cssText = generateCSS(leftGuide, top + height / 2, leftDistance);
            leftRef.current.dataset.distance = `${leftDistance}px`;
        }
        else {
            leftRef.current.style.display = 'none';
        }
        if (rightDistance !== 0) {
            rightRef.current.style.cssText = generateCSS(rightGuide, top + height / 2, rightDistance);
            rightRef.current.dataset.distance = `${rightDistance}px`;
        }
        else {
            rightRef.current.style.display = 'none';
        }
        if (topDistance !== 0) {
            topRef.current.style.cssText = generateCSS(left + width / 2, topGuide, undefined, topDistance);
            topRef.current.dataset.distance = `${topDistance}px`;
        }
        else {
            topRef.current.style.display = 'none';
        }
        if (bottomDistance !== 0) {
            bottomRef.current.style.cssText = generateCSS(left + width / 2, bottomGuide, undefined, bottomDistance);
            bottomRef.current.dataset.distance = `${bottomDistance}px`;
        }
        else {
            bottomRef.current.style.display = 'none';
        }
        setPosition([leftRef.current, rightRef.current, topRef.current, bottomRef.current], isModal);
    }
    else if (leftRef.current) {
        leftRef.current.style.display = 'none';
        rightRef.current.style.display = 'none';
        topRef.current.style.display = 'none';
        bottomRef.current.style.display = 'none';
    }
    return (React__default.createElement(React__default.Fragment, null,
        React__default.createElement("div", { ref: topRef, className: styles$2['distances-v'] }),
        React__default.createElement("div", { ref: bottomRef, className: styles$2['distances-v'] }),
        React__default.createElement("div", { ref: leftRef, className: styles$2['distances-h'] }),
        React__default.createElement("div", { ref: rightRef, className: styles$2['distances-h'] })));
}
var Distances$1 = React.memo(Distances);

var css_248z$3 = ".index_animate__2FQ5E {\n  transition: all 50ms;\n}\n.index_distances-tip__3U4_J {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n}\n.index_border-container__3T4fr {\n  position: absolute;\n  z-index: 20000;\n  border: 1px #00B37A dashed;\n  pointer-events: none;\n  display: none;\n  justify-content: center;\n  align-items: center;\n  transition: all 100ms;\n}\n.index_baseboard__1y_uO {\n  position: absolute;\n  z-index: 10000;\n  top: 0;\n  left: 0;\n  height: 100%;\n  width: 100%;\n  background-color: #00000000;\n  display: none;\n  pointer-events: auto;\n}\n.index_size-tip-width__3ERhN {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n  pointer-events: auto;\n  top: 5px;\n}\n.index_size-tip-height__2eDHc {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n  pointer-events: auto;\n  left: 5px;\n}\n.index_tip-hidden__2DQkF {\n  display: none;\n}\n.index_resize-item__1gVBD {\n  position: absolute;\n  height: 7px;\n  width: 7px;\n  border-radius: 50%;\n  background-color: white;\n  border: 1px #00B37A dashed;\n  pointer-events: auto;\n}\n";
var styles$3 = {"animate":"index_animate__2FQ5E","distances-tip":"index_distances-tip__3U4_J","border-container":"index_border-container__3T4fr","baseboard":"index_baseboard__1y_uO","size-tip-width":"index_size-tip-width__3ERhN","size-tip-height":"index_size-tip-height__2eDHc","tip-hidden":"index_tip-hidden__2DQkF","resize-item":"index_resize-item__1gVBD"};
styleInject(css_248z$3);

var css_248z$4 = ".index_animate__29Hr5 {\n  transition: all 50ms;\n}\n.index_distances-tip__HO_ig {\n  position: absolute;\n  display: block;\n  font-size: 12px;\n  color: #FFF;\n  line-height: 12px;\n  padding: 3px;\n  border-radius: 2px;\n  z-index: 1;\n  background-color: #00B37A;\n  transition: all 50ms;\n}\n.index_container__2XofR {\n  position: absolute;\n  z-index: 1;\n  display: flex;\n  right: 2px;\n  flex-direction: row;\n}\n.index_action-btn__ujM3x {\n  width: 32px;\n  height: 32px;\n  display: flex;\n  justify-content: center;\n  align-items: center;\n  pointer-events: auto;\n  margin-left: 3px;\n  margin-right: 3px;\n  background: #fafbfc;\n  border: 1px solid #ebecf0;\n}\n.index_action-icon__1Sop1 {\n  width: 20px;\n  height: 20px;\n}\n";
var styles$4 = {"animate":"index_animate__29Hr5","distances-tip":"index_distances-tip__HO_ig","container":"index_container__2XofR","action-btn":"index_action-btn__ujM3x","action-icon":"index_action-icon__1Sop1"};
styleInject(css_248z$4);

const img = "data:image/svg+xml,%3c%3fxml version='1.0' encoding='UTF-8'%3f%3e%3csvg width='16px' height='16px' viewBox='0 0 16 16' version='1.1' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3e %3ctitle%3e2-Edit/edit%3c/title%3e %3cdefs%3e %3cpolygon id='path-1' points='0 20 20 20 20 0 0 0'%3e%3c/polygon%3e %3cpath d='M16.162%2c5.0843 L14.904%2c3.8263 L15.522%2c3.2093 L16.816%2c4.4283 L16.162%2c5.0843 L16.162%2c5.0843 Z M10.329%2c10.9353 L8.62%2c11.3633 L9.024%2c9.7063 L14.055%2c4.6753 L15.315%2c5.9343 L10.329%2c10.9353 L10.329%2c10.9353 Z M10.1077%2c3.5642 C10.4377%2c3.5642 10.7077%2c3.8342 10.7077%2c4.1642 C10.7077%2c4.4942 10.4377%2c4.7642 10.1077%2c4.7642 L10.1077%2c4.7642 L3.9997%2c4.7642 C3.5667%2c4.7642 3.1997%2c5.1302 3.1997%2c5.5642 L3.1997%2c5.5642 L3.1997%2c16.0002 C3.1997%2c16.4332 3.5667%2c16.8002 3.9997%2c16.8002 L3.9997%2c16.8002 L14.4357%2c16.8002 C14.8697%2c16.8002 15.2357%2c16.4332 15.2357%2c16.0002 L15.2357%2c16.0002 L15.2357%2c9.5922 C15.2357%2c9.2622 15.5057%2c8.9922 15.8357%2c8.9922 C16.1657%2c8.9922 16.4357%2c9.2622 16.4357%2c9.5922 L16.4357%2c9.5922 L16.4357%2c16.0002 C16.4357%2c17.1002 15.5357%2c18.0002 14.4357%2c18.0002 L14.4357%2c18.0002 L3.9997%2c18.0002 C2.8997%2c18.0002 1.9997%2c17.1002 1.9997%2c16.0002 L1.9997%2c16.0002 L1.9997%2c5.5642 C1.9997%2c4.4642 2.8997%2c3.5642 3.9997%2c3.5642 L3.9997%2c3.5642 Z M14.711%2c2.3223 C15.143%2c1.8893 15.898%2c1.8903 16.33%2c2.3223 L16.33%2c2.3223 L17.665%2c3.6573 C18.11%2c4.1033 18.111%2c4.8293 17.665%2c5.2763 L17.665%2c5.2763 L11.129%2c11.8113 C10.988%2c11.9533 10.811%2c12.0523 10.618%2c12.1003 L10.618%2c12.1003 L7.947%2c12.7683 C7.899%2c12.7803 7.85%2c12.7863 7.801%2c12.7863 C7.644%2c12.7863 7.491%2c12.7243 7.377%2c12.6103 C7.228%2c12.4613 7.168%2c12.2453 7.219%2c12.0403 L7.219%2c12.0403 L7.887%2c9.3673 C7.936%2c9.1753 8.035%2c9.0003 8.174%2c8.8593 L8.174%2c8.8593 Z' id='path-3'%3e%3c/path%3e %3c/defs%3e %3cg id='R.-%e5%b9%b3%e5%8f%b0%e9%a1%b5%e9%9d%a2%e6%a8%a1%e6%9d%bf' stroke='none' stroke-width='1' fill='none' fill-rule='evenodd'%3e %3cg id='R.1.2-%e7%bc%96%e8%be%91%e5%95%86%e5%9f%8e%e6%a8%a1%e6%9d%bf-1' transform='translate(-912.000000%2c -820.000000)'%3e %3cg id='%e7%bc%96%e8%be%91' transform='translate(666.000000%2c 796.000000)'%3e %3cg id='Button/Icon/Secondary/Small/01-Normal' transform='translate(238.000000%2c 16.000000)'%3e %3cg id='icon-edit' transform='translate(6.000000%2c 6.000000)'%3e %3cmask id='mask-2' fill='white'%3e %3cuse xlink:href='%23path-1'%3e%3c/use%3e %3c/mask%3e %3cg%3e%3c/g%3e %3cmask id='mask-4' fill='white'%3e %3cuse xlink:href='%23path-3'%3e%3c/use%3e %3c/mask%3e %3cuse fill='%23A6A6A6' xlink:href='%23path-3'%3e%3c/use%3e %3crect id='Color' fill='%236B778C' mask='url(%23mask-4)' x='0' y='0' width='20' height='20'%3e%3c/rect%3e %3c/g%3e %3c/g%3e %3c/g%3e %3c/g%3e %3c/g%3e%3c/svg%3e";

// import { clearChildNodes, copyComponent, deleteComponent } from 'lingxi-editor-core';
const ACTIONS = {
    delete: 'delete',
    copy: 'copy',
    clear: 'clear',
    edit: 'edit'
};
const configs = [
    // {
    //   type: ACTIONS.delete,
    //   action: deleteComponent,
    //   icon: deleteSvg
    // },
    // {
    //   type: ACTIONS.copy,
    //   action: copyComponent,
    //   icon: copySvg
    // },
    // {
    //   type: ACTIONS.clear,
    //   action: clearChildNodes,
    //   icon: clearSvg
    // },
    {
        type: ACTIONS.edit,
        action: () => null,
        icon: img
    }
];

function ActionSheet(props) {
    const { isOut, isRoot, hasChildNodes } = props;
    return (React__default.createElement("div", { className: styles$4['container'], style: { top: isOut ? 16 : 0 } }, configs.map((config) => {
        const { icon, action, type } = config;
        if (isRoot && type === ACTIONS.copy)
            return null;
        if (!hasChildNodes && type === ACTIONS.clear)
            return null;
        return (React__default.createElement("div", { className: styles$4['action-btn'], onClick: action, key: type },
            React__default.createElement("img", { src: icon, className: styles$4['action-icon'] })));
    })));
}
var ActionSheet$1 = React.memo(ActionSheet);

var Direction;
(function (Direction) {
    Direction["top"] = "top";
    Direction["right"] = "right";
    Direction["bottom"] = "bottom";
    Direction["left"] = "left";
    Direction["topRight"] = "topRight";
    Direction["bottomRight"] = "bottomRight";
    Direction["bottomLeft"] = "bottomLeft";
    Direction["topLeft"] = "topLeft";
})(Direction || (Direction = {}));
const controlUpdate$3 = (prevState, nextState) => {
    const { selectedInfo, componentConfigs, hoverKey } = nextState;
    return prevState.selectedInfo !== selectedInfo ||
        selectedInfo && (prevState.componentConfigs !== componentConfigs ||
            prevState.hoverKey === null && hoverKey !== null ||
            prevState.hoverKey !== null && hoverKey === null);
};
function Resize() {
    const iframe = getIframe();
    const [hideActionSheet] = React.useState(true);
    const [hideSizeTip] = React.useState(true);
    const { selectedInfo, hoverKey, componentConfigs } = lingxiEditorCore.useSelector(['selectedInfo', 'componentConfigs', 'hoverKey'], controlUpdate$3);
    const { selectedKey, domTreeKeys, propName } = selectedInfo || {};
    const resizeRef = React.useRef();
    const originSizeRef = React.useRef();
    const sizeResultRef = React.useRef({});
    const widthRef = React.useRef();
    const heightRef = React.useRef();
    const baseboardRef = React.useRef();
    const selectNodeRef = React.useRef();
    const [isOut, setIsOut] = React.useState(true);
    const { props, childNodes } = componentConfigs[selectedKey] || {};
    const { width = 'auto', height = 'auto' } = lodash.get(props, 'style', {});
    const isModal = React.useMemo(() => getIsModalChild(componentConfigs, domTreeKeys), [domTreeKeys, componentConfigs]);
    selectNodeRef.current = React.useMemo(() => getSelectedNode(selectedKey, iframe), [iframe, selectedKey]);
    React.useEffect(() => {
        const contentWindow = iframe.contentWindow;
        const changeSize = () => setSelectedBorder();
        contentWindow.addEventListener('mouseup', onMouseUp);
        contentWindow.addEventListener('mousemove', onMouseMove);
        contentWindow.addEventListener('mouseleave', onMouseUp);
        contentWindow.addEventListener('resize', changeSize);
        contentWindow.addEventListener('animationend', changeSize);
        return () => {
            contentWindow.removeEventListener('mouseup', onMouseUp);
            contentWindow.removeEventListener('mousemove', onMouseMove);
            contentWindow.removeEventListener('mousemove', onMouseUp);
            contentWindow.removeEventListener('resize', changeSize);
            contentWindow.removeEventListener('animationend', changeSize);
        };
    }, [isModal]);
    React.useEffect(() => {
        if (selectedKey) {
            if (selectNodeRef.current) {
                selectNodeRef.current = getSelectedNode(selectedKey, iframe);
                setSelectedBorder();
            }
            else {
                setSelectedBorder();
            }
        }
    }, [selectedKey, selectNodeRef.current, iframe, componentConfigs]);
    if (!selectedKey && resizeRef.current) {
        resizeRef.current.style.display = 'none';
    }
    const onMouseUp = React.useCallback(() => {
        originSizeRef.current = undefined;
        baseboardRef.current.style.display = 'none';
        resizeRef.current.style.pointerEvents = 'none';
        resizeRef.current.style.transitionProperty = 'all';
        resizeRef.current.style.transitionDuration = '50ms';
        lingxiEditorCore.resizeChange(sizeResultRef.current);
        sizeResultRef.current = {};
    }, [originSizeRef.current, baseboardRef.current, resizeRef.current, sizeResultRef.current]);
    const changeBaseboard = () => {
        const { body: { scrollWidth, scrollHeight } } = iframe.contentDocument;
        baseboardRef.current.style.cssText = `
    display:block;
    width:${scrollWidth}px;
    height:${scrollHeight}px
    `;
    };
    const onMouseMove = React.useCallback((event) => {
        event.stopPropagation();
        if (originSizeRef.current) {
            const { clientX, clientY } = event;
            const { x, y, direction, height, width } = originSizeRef.current;
            let offsetY = 0;
            let offsetX = 0;
            switch (direction) {
                case Direction.left:
                    offsetX = x - clientX;
                    break;
                case Direction.right:
                    offsetX = clientX - x;
                    break;
                case Direction.top:
                    offsetY = y - clientY;
                    break;
                case Direction.bottom:
                    offsetY = clientY - y;
                    break;
                case Direction.topLeft:
                    offsetY = y - clientY;
                    offsetX = x - clientX;
                    break;
                case Direction.topRight:
                    offsetY = y - clientY;
                    offsetX = clientX - x;
                    break;
                case Direction.bottomLeft:
                    offsetX = x - clientX;
                    offsetY = clientY - y;
                    break;
                case Direction.bottomRight:
                    offsetY = clientY - y;
                    offsetX = clientX - x;
                    break;
            }
            const heightResult = height + offsetY;
            const widthResult = width + offsetX;
            const { minWidth, maxHeight, maxWidth, minHeight } = originSizeRef.current;
            if (offsetX !== 0 && (minWidth === null || widthResult >= minWidth) && (maxWidth === null || widthResult <= maxWidth)) {
                sizeResultRef.current.width = `${widthResult}px`;
                selectNodeRef.current.style.width = `${widthResult}px`;
            }
            if (offsetY !== 0 && (minHeight === null || heightResult >= minHeight) && (maxHeight === null || heightResult <= maxHeight)) {
                sizeResultRef.current.height = `${heightResult}px`;
                selectNodeRef.current.style.height = `${heightResult}px`;
            }
            showSize(sizeResultRef.current.width, sizeResultRef.current.height);
            setSelectedBorder('pointer-events: auto; transition:none;');
            changeBaseboard();
        }
    }, [originSizeRef.current, sizeResultRef.current, selectNodeRef.current]);
    const showSize = React.useCallback((width, height) => {
        if (width) {
            widthRef.current.innerHTML = width;
        }
        if (height) {
            heightRef.current.innerHTML = height;
        }
    }, [heightRef.current, widthRef.current]);
    const setSelectedBorder = (css = '') => {
        if (selectNodeRef.current) {
            const { left, top, width, height } = getElementInfo(selectNodeRef.current, iframe, isModal);
            if (top <= 14 && isOut) {
                setIsOut(false);
            }
            else if (top > 14 && !isOut) {
                setIsOut(true);
            }
            resizeRef.current.style.cssText = generateCSS(left, top, width, height) + css;
            setPosition([resizeRef.current], isModal);
        }
    };
    // const onResizeStart = useCallback(function (event: React.MouseEvent<HTMLSpanElement>, direction: Direction) {
    //   if (event.nativeEvent && iframe) {
    //     const { contentWindow } = iframe!;
    //     const { width, height, minWidth, minHeight, maxWidth, maxHeight } = contentWindow!.getComputedStyle(selectNodeRef.current!);
    //     originSizeRef.current = {
    //       x: event.nativeEvent.clientX,
    //       y: event.nativeEvent.clientY,
    //       direction,
    //       width: formatUnit(width)!,
    //       height: formatUnit(height)!,
    //       minWidth: formatUnit(minWidth),
    //       minHeight: formatUnit(minHeight),
    //       maxWidth: formatUnit(maxWidth),
    //       maxHeight: formatUnit(maxHeight),
    //     };
    //     changeBaseboard()
    //   }
    // }, [iframe, selectNodeRef.current, resizeRef.current, baseboardRef.current, originSizeRef.current]);
    const changeWidth = React.useCallback(() => {
        widthRef.current.contentEditable = 'true';
    }, [widthRef.current]);
    const changeHeight = React.useCallback(() => {
        heightRef.current.contentEditable = 'true';
    }, [heightRef.current]);
    /**
     * 更新组件宽高
     */
    const updateSize = React.useCallback(() => {
        const width = widthRef.current.innerHTML;
        const height = heightRef.current.innerHTML;
        lingxiEditorCore.resizeChange({ width, height });
        widthRef.current.contentEditable = 'false';
        heightRef.current.contentEditable = 'false';
    }, [widthRef.current, heightRef.current]);
    setSelectedBorder();
    return (React__default.createElement(React__default.Fragment, null,
        React__default.createElement("div", { className: styles$3['border-container'], ref: resizeRef },
            React__default.createElement(ActionSheet$1, { isOut: isOut, hasChildNodes: propName ? !!lodash.get(childNodes, propName) : !!childNodes, isRoot: selectedKey === lingxiEditorCore.ROOT }),
            !hideActionSheet && React__default.createElement(ActionSheet$1, { isOut: isOut, hasChildNodes: propName ? !!lodash.get(childNodes, propName) : !!childNodes, isRoot: selectedKey === lingxiEditorCore.ROOT }),
            !hideSizeTip && (React__default.createElement(React.Fragment, null,
                React__default.createElement("div", { className: hoverKey ? styles$3['tip-hidden'] : styles$3['size-tip-width'], ref: widthRef, onDoubleClick: updateSize, onClick: changeWidth }, width),
                React__default.createElement("div", { className: hoverKey ? styles$3['tip-hidden'] : styles$3['size-tip-height'], ref: heightRef, onDoubleClick: updateSize, onClick: changeHeight }, height)))),
        React__default.createElement("div", { ref: baseboardRef, className: styles$3['baseboard'] })));
}
var Resize$1 = React.memo(Resize);

const onIframeLoad = (divContainer, designPage, iframe) => {
    const head = document.head.cloneNode(true);
    const contentDocument = iframe.contentDocument;
    contentDocument.head.remove();
    contentDocument.documentElement.insertBefore(head, contentDocument.body);
    divContainer.current = contentDocument.getElementById('dnd-container');
    componentMount(designPage, divContainer);
};
const componentMount = (designPage, divContainer) => {
    ReactDOM.render(React__default.createElement(lingxiEditorCore.LegoProvider, null,
        designPage,
        React__default.createElement(Guidelines$1, null),
        React__default.createElement(Distances$1, null),
        React__default.createElement(Resize$1, null)), divContainer.current);
};
const onDragEnter = (dragSource, divContainer) => {
    const { vDOMCollection } = dragSource;
    componentMount(renderComponent(vDOMCollection), divContainer);
};
const onDragLeave = (divContainer) => {
    ReactDOM.unmountComponentAtNode(divContainer.current);
};
const renderComponent = (componentConfigs) => {
    const { [lingxiEditorCore.ROOT]: { componentName } } = componentConfigs;
    const props = {
        specialProps: {
            domTreeKeys: [lingxiEditorCore.ROOT],
            key: lingxiEditorCore.ROOT,
            parentKey: '',
            draggable: false
        },
    };
    return lingxiEditorCore.isContainer(componentName) ? React__default.createElement(Container$1, Object.assign({ onMouseLeave: lingxiEditorCore.clearHovered }, props)) :
        React__default.createElement(NoneContainer$1, Object.assign({ onMouseLeave: lingxiEditorCore.clearHovered }, props));
};
const stateSelector$1 = ['componentConfigs', 'dragSource'];
const controlUpdate$4 = (prevState, nextState) => {
    const { componentConfigs: prevComponentConfigs } = prevState;
    const { componentConfigs } = nextState;
    return !componentConfigs[lingxiEditorCore.ROOT] || componentConfigs[lingxiEditorCore.ROOT] !== prevComponentConfigs[lingxiEditorCore.ROOT];
};
function BrickDesign(props) {
    const { componentConfigs, dragSource } = lingxiEditorCore.useSelector(stateSelector$1, controlUpdate$4);
    const iframeRef = React.useRef();
    const designPage = React.useMemo(() => {
        if (!componentConfigs[lingxiEditorCore.ROOT])
            return null;
        return renderComponent(componentConfigs);
    }, [componentConfigs]);
    const divContainer = React.useRef(null);
    React.useEffect(() => {
        iframeRef.current = getIframe();
        const contentWindow = iframeRef.current.contentWindow;
        contentWindow.addEventListener('dragover', onDragover);
        contentWindow.addEventListener('drop', onDrop);
        return () => {
            contentWindow.removeEventListener('dragover', onDragover);
            contentWindow.removeEventListener('drop', onDrop);
        };
    }, []);
    React.useEffect(() => {
        if (componentConfigs[lingxiEditorCore.ROOT])
            return;
        const contentWindow = getIframe().contentWindow;
        const dragEnter = () => onDragEnter(dragSource, divContainer);
        const dragLeave = () => onDragLeave(divContainer);
        contentWindow.addEventListener('dragenter', dragEnter);
        contentWindow.addEventListener('dragleave', dragLeave);
        return () => {
            contentWindow.removeEventListener('dragenter', dragEnter);
            contentWindow.removeEventListener('dragleave', dragLeave);
        };
    }, [componentConfigs, dragSource, divContainer, iframeRef]);
    React.useEffect(() => {
        if (divContainer.current) {
            componentMount(designPage, divContainer);
        }
    }, [divContainer.current, designPage]);
    const { onLoadEnd } = props;
    return (React__default.createElement("iframe", Object.assign({ id: "dnd-iframe", style: { border: 0, width: '100%', height: '100%' }, srcDoc: iframeSrcDoc, onLoad: React.useCallback(() => {
            onIframeLoad(divContainer, designPage, iframeRef.current);
            onLoadEnd && onLoadEnd();
        }, [iframeRef.current]) }, props)));
}
var BrickDesign$1 = React.memo(BrickDesign);

exports.BrickDesign = BrickDesign$1;
exports.SPECIAL_STRING_CONSTANTS = SPECIAL_STRING_CONSTANTS;
exports.formatSpecialProps = formatSpecialProps;
exports.formatUnit = formatUnit;
exports.generateCSS = generateCSS;
exports.generateRequiredProps = generateRequiredProps;
exports.getComponent = getComponent;
exports.getDropTargetInfo = getDropTargetInfo;
exports.getElementInfo = getElementInfo;
exports.getIframe = getIframe;
exports.getIsModalChild = getIsModalChild;
exports.getSelectedNode = getSelectedNode;
exports.handleSelectedStatus = handleSelectedStatus;
exports.iframeSrcDoc = iframeSrcDoc;
exports.isEqualKey = isEqualKey;
exports.onDragStart = onDragStart;
exports.onDragover = onDragover;
exports.onDrop = onDrop;
exports.onMouseOver = onMouseOver;
exports.selectedStatus = selectedStatus;
exports.setPosition = setPosition;
exports.usePrevious = usePrevious;
