import React, { memo, useCallback, useEffect, useMemo, useRef } from 'react';
import { clearHovered, isContainer, LegoProvider, ROOT, useSelector, } from 'lingxi-editor-core';
import ReactDOM from 'react-dom';
import Container from './warppers/Container';
import NoneContainer from './warppers/NoneContainer';
import { getIframe, iframeSrcDoc } from './utils';
import { onDragover, onDrop } from './common/events';
import Guidelines from './components/Guidelines';
import Distances from './components/Distances';
import Resize from './components/Resize';
const onIframeLoad = (divContainer, designPage, iframe) => {
    const head = document.head.cloneNode(true);
    const contentDocument = iframe.contentDocument;
    contentDocument.head.remove();
    contentDocument.documentElement.insertBefore(head, contentDocument.body);
    divContainer.current = contentDocument.getElementById('dnd-container');
    componentMount(designPage, divContainer);
};
const componentMount = (designPage, divContainer) => {
    ReactDOM.render(React.createElement(LegoProvider, null,
        designPage,
        React.createElement(Guidelines, null),
        React.createElement(Distances, null),
        React.createElement(Resize, null)), divContainer.current);
};
const onDragEnter = (dragSource, divContainer) => {
    const { vDOMCollection } = dragSource;
    componentMount(renderComponent(vDOMCollection), divContainer);
};
const onDragLeave = (divContainer) => {
    ReactDOM.unmountComponentAtNode(divContainer.current);
};
const renderComponent = (componentConfigs) => {
    const { [ROOT]: { componentName } } = componentConfigs;
    const props = {
        specialProps: {
            domTreeKeys: [ROOT],
            key: ROOT,
            parentKey: '',
            draggable: false
        },
    };
    return isContainer(componentName) ? React.createElement(Container, Object.assign({ onMouseLeave: clearHovered }, props)) :
        React.createElement(NoneContainer, Object.assign({ onMouseLeave: clearHovered }, props));
};
const stateSelector = ['componentConfigs', 'dragSource'];
const controlUpdate = (prevState, nextState) => {
    const { componentConfigs: prevComponentConfigs } = prevState;
    const { componentConfigs } = nextState;
    return !componentConfigs[ROOT] || componentConfigs[ROOT] !== prevComponentConfigs[ROOT];
};
function BrickDesign(props) {
    const { componentConfigs, dragSource } = useSelector(stateSelector, controlUpdate);
    const iframeRef = useRef();
    const designPage = useMemo(() => {
        if (!componentConfigs[ROOT])
            return null;
        return renderComponent(componentConfigs);
    }, [componentConfigs]);
    const divContainer = useRef(null);
    useEffect(() => {
        iframeRef.current = getIframe();
        const contentWindow = iframeRef.current.contentWindow;
        contentWindow.addEventListener('dragover', onDragover);
        contentWindow.addEventListener('drop', onDrop);
        return () => {
            contentWindow.removeEventListener('dragover', onDragover);
            contentWindow.removeEventListener('drop', onDrop);
        };
    }, []);
    useEffect(() => {
        if (componentConfigs[ROOT])
            return;
        const contentWindow = getIframe().contentWindow;
        const dragEnter = () => onDragEnter(dragSource, divContainer);
        const dragLeave = () => onDragLeave(divContainer);
        contentWindow.addEventListener('dragenter', dragEnter);
        contentWindow.addEventListener('dragleave', dragLeave);
        return () => {
            contentWindow.removeEventListener('dragenter', dragEnter);
            contentWindow.removeEventListener('dragleave', dragLeave);
        };
    }, [componentConfigs, dragSource, divContainer, iframeRef]);
    useEffect(() => {
        if (divContainer.current) {
            componentMount(designPage, divContainer);
        }
    }, [divContainer.current, designPage]);
    const { onLoadEnd } = props;
    return (React.createElement("iframe", Object.assign({ id: "dnd-iframe", style: { border: 0, width: '100%', height: '100%' }, srcDoc: iframeSrcDoc, onLoad: useCallback(() => {
            onIframeLoad(divContainer, designPage, iframeRef.current);
            onLoadEnd && onLoadEnd();
        }, [iframeRef.current]) }, props)));
}
export default memo(BrickDesign);
