import { addComponent, clearHovered, clearSelectedStatus, getDragSource, getDropTarget, overTarget, selectComponent, } from 'lingxi-editor-core';
import { isEqualKey } from '../utils';
export function handleSelectedStatus(event, isSelected, specialProps, propName) {
    event && event.stopPropagation && event.stopPropagation();
    if (isSelected) {
        clearSelectedStatus();
    }
    else {
        selectComponent({ ...specialProps, propName });
    }
}
/**
 * hover组件上触发
 * @param event
 * @param key
 */
export function onMouseOver(event, key, isSelected) {
    event && event.stopPropagation && event.stopPropagation();
    if (isSelected) {
        clearHovered();
    }
    else {
        overTarget({
            hoverKey: key,
        });
    }
}
/**
 * 获取要放入组件的容器信息
 * @param event
 * @param domTreeKeys
 * @param selectedKey
 * @param propName
 */
export function getDropTargetInfo(event, domTreeKeys, selectedKey, propName) {
    event && event.stopPropagation && event.stopPropagation();
    getDropTarget({
        propName,
        selectedKey,
        domTreeKeys,
    });
}
/**
 * 拖拽当前组件时获取当前组件的信息
 * @param event
 * @param dragKey
 * @param parentKey
 * @param parentPropName
 */
export function onDragStart(event, dragKey, parentKey, parentPropName) {
    event && event.stopPropagation && event.stopPropagation();
    setTimeout(() => {
        getDragSource({
            dragKey,
            parentKey,
            parentPropName,
        });
    }, 0);
}
/**
 * 获取组件选中状态
 * @param key
 * @param hoverKey
 * @param selectedKey
 */
export function selectedStatus(key, hoverKey, selectedKey) {
    const isSelected = isEqualKey(key, selectedKey);
    /** 是否hover到当前组件 */
    const isHovered = isEqualKey(key, hoverKey);
    return { isHovered, isSelected };
}
export function onDragover(e) {
    e.preventDefault();
}
export function onDrop(e) {
    e.stopPropagation();
    addComponent();
}
