import React, { memo, useEffect, useRef } from 'react';
import styles from './index.less';
import { useSelector, } from 'lingxi-editor-core';
import { generateCSS, getElementInfo, getIframe, getIsModalChild, getSelectedNode, setPosition } from '../../utils';
const controlUpdate = () => {
    return true;
};
function Guidelines() {
    const topRef = useRef();
    const bottomRef = useRef();
    const leftRef = useRef();
    const rightRef = useRef();
    const hoverNodeRef = useRef();
    const iframe = getIframe();
    const { hoverKey, dropTarget, dragSource, selectedInfo, componentConfigs } = useSelector(['hoverKey', 'dropTarget', 'dragSource', 'selectedInfo', 'componentConfigs'], controlUpdate);
    const { domTreeKeys } = selectedInfo || {};
    const isModal = getIsModalChild(componentConfigs, domTreeKeys);
    const guidControl = hoverKey && (!selectedInfo || selectedInfo && !dragSource);
    useEffect(() => {
        const { contentWindow } = iframe;
        const renderGuideLines = () => {
            const node = getSelectedNode(hoverKey, iframe);
            if (guidControl && node) {
                const { left, top, bottom, right, width, height } = getElementInfo(node, iframe, isModal);
                const { contentDocument } = iframe;
                hoverNodeRef.current.style.cssText = generateCSS(left, top, width, height);
                topRef.current.style.top = `${top}px`;
                topRef.current.style.width = `${contentDocument.body.scrollWidth}px`;
                leftRef.current.style.left = `${left}px`;
                leftRef.current.style.height = `${contentDocument.body.scrollHeight}px`;
                rightRef.current.style.left = `${right - 1}px`;
                rightRef.current.style.height = `${contentDocument.body.scrollHeight}px`;
                bottomRef.current.style.top = `${bottom - 1}px`;
                bottomRef.current.style.width = `${contentDocument.body.scrollWidth}px`;
                setPosition([hoverNodeRef.current, leftRef.current, rightRef.current, topRef.current, bottomRef.current], isModal);
            }
        };
        renderGuideLines();
        const onScroll = () => {
            setTimeout(renderGuideLines, 66);
        };
        contentWindow.addEventListener('scroll', onScroll);
        return () => {
            contentWindow.removeEventListener('scroll', onScroll);
        };
    });
    const guidH = guidControl ? styles['guide-h'] : styles['guide-hidden'];
    const guidV = guidControl ? styles['guide-v'] : styles['guide-hidden'];
    const hoverNode = guidControl ? dropTarget ? styles['drop-node'] : styles['hover-node'] : styles['guide-hidden'];
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { ref: hoverNodeRef, className: hoverNode }),
        React.createElement("div", { ref: leftRef, className: guidV }),
        React.createElement("div", { ref: rightRef, className: guidV }),
        React.createElement("div", { ref: topRef, className: guidH }),
        React.createElement("div", { ref: bottomRef, className: guidH })));
}
export default memo(Guidelines);
